{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Backups" %}{% endblock %}

{% block page_header %}
<base href="{{ WEBROOT }}">
{% endblock page_header %}

{% block main %}
<ng-include src="'{{STATIC_URL}}/dashboard/project/ngbackups/table/table.html'"></ng-include>
{% endblock %}
