{% extends "horizon/common/_modal.html" %}
{% load i18n %}

{% block modal-header %}{% trans "Log Contents" %}{% endblock %}

{% block modal-body %}
<pre class="logs">
  {{ log.data }}
</pre>
{% endblock %}

{% block modal-footer %}
  <a href="{% url 'horizon:project:databases:index' %}" class="btn btn-default secondary cancel close">{% trans "Close" %}</a>
{% endblock %}
