{% load i18n %}

<noscript><h3>{{ step }}</h3></noscript>
<div id="networkListSortContainer" class="sort-container">
  <div class="col-sm-6">
    <label id="selected_network_label">{% trans "Selected networks" %}</label>
    <ul id="selected_network" class="networklist box-list"></ul>
    <label>{% trans "Available networks" %}</label>
    <ul id="available_network" class="networklist box-list"></ul>
  </div>
  <div class="col-sm-6">
    {% include "project/databases/_launch_instances_network_help.html" %}
  </div>
</div>

<div id="networkListIdContainer">
  <div class="actions">
    <div id="networkListId">
      {% include "horizon/common/_form_fields.html" %}
    </div>
  </div>
  <div class="help_text">
    {{ step.get_help_text }}
  </div>
</div>


<script>
  if (typeof $ !== 'undefined') {
    horizon.instances.workflow_init($(".workflow"));
  } else {
    addHorizonLoadEvent(function() {
      horizon.instances.workflow_init($(".workflow"));
    });
  }
</script>

