{% load i18n %}

<h4>{% trans "Initial Databases" %}</h4>
<p>{% trans "Optionally provide a comma separated list of databases to create:" %}</p>
<pre>database1, database2, database3</pre>

<h4>{% trans "Initial Admin User" %}</h4>
<p>{% blocktrans trimmed %}Create an optional initial user.
   This user will have access to all databases you create.{% endblocktrans %}</p>

<ul>
  <li><strong>{% trans "Username (required)" %}</strong></li>
  <li><strong>{% trans "Password (required)" %}</strong></li>
  <li><strong>{% trans "Allowed Host (optional)" %}</strong>
    {% blocktrans trimmed %}Allow the user to connect from this host
    only. If not provided this user will be allowed to connect from anywhere.
    {% endblocktrans %}</li>
</ul>
