{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block form_id %}reset_password_form{% endblock %}
{% block form_action %}{% url "horizon:project:database_clusters:reset_password" cluster_id %}{% endblock %}

{% block modal_id %}reset_password_modal{% endblock %}
{% block modal-header %}{% trans "Reset Root Password" %}{% endblock %}

{% block modal-body %}
<div class="left">
  <fieldset>
  {% include "horizon/common/_form_fields.html" %}
  </fieldset>
</div>
<div class="right">
  <p>{% blocktrans trimmed %}Specify the new root password for vertica cluster.{% endblocktrans %}</p>
</div>
{% endblock %}

{% block modal-footer %}
  <input class="btn btn-primary pull-right" type="submit" value="{% trans "Reset Root Password" %}" />
  <a href="{% url "horizon:project:database_clusters:index" %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
