{% load i18n %}

<br>
<div class="help_text">
  {% trans "Manage parameters of the configuration group. If the configuration group is attached with database instance, the change is applied to the instance synchronously." %}
</div>
<br>
{{ table.render }}
