{% load i18n %}

<p>{% blocktrans trimmed %}Access defines how the database service is exposed.{% endblocktrans %}</p>
<p>{% blocktrans trimmed %}CIDRs is a comma-separated list of IPv4, IPv6 or mix of both CIDRs that restrict access to the database service. 0.0.0.0/0 is used by default if this parameter is not provided. E.g.: 0.0.0.0/0,202.78.240.0/24{% endblocktrans %}</p>
